# -*- coding: utf-8 -*-

from kinder.core.direct.models import Direct
from kinder.core.group.models import Group
from kinder.core.group.models import GroupStatusEnum
from kinder.core.group.models import Pupil

from . import BaseIndex
from ..algorithm.direct import *
from ..constants import ALL
from ..utils import AgeDeltas
from .enrolled import Index


class DirectIndex(BaseIndex):
    """Запросы расчитывающие количество направлений,
    написанные на django queryset'ах"""

    FILTER_193 = 1
    FILTER_291 = 2
    FILTER_32 = 3

    ALGORITHM_MAP = {
        FILTER_193: get_193_filter,
        FILTER_291: get_291_filter,
        FILTER_32: get_34_filter,
    }

    def __init__(self, unit):
        self.unit = unit

    def get_count(self, filter_type, **kwargs):

        qs = Direct.objects.filter(
            self.get_algorithm(filter_type)(
                unit=self.unit, **kwargs))

        age_range = kwargs.get('age_range', ALL)
        if age_range != ALL:
            on_date = kwargs.get('on_date', AgeDeltas.calculate_date)
            down, up = AgeDeltas.get_category_deltas(age_range, on_date)
            qs = qs.filter(
                declaration__children__date_of_birth__gt=down,
                declaration__children__date_of_birth__lte=up)

        return qs.filter(group__unit=self.unit).count()


def ind_19_3(dou, age_range=ALL):
    u"""Показатель #19.3."""

    result = 0
    group_index = Index(dou)
    for group in Group.objects.filter(
            status=GroupStatusEnum.FACT, unit=dou):
        for _, date_of_birth in group_index.directs_in_group(group.id):
            down, up = AgeDeltas.get_category_deltas(
                age_range, datetime.date.today)
            if down < date_of_birth <= up:
                result += 1

    return result


def ind_29_1(dou):
    u"""Показатель #29.1 Прогнозируемое уменьшение контингента."""

    groups = dou.group_set.filter(status=GroupStatusEnum.FACT)
    children_ids = set()

    for group in groups.iterator():
        qs = Pupil.objects.filter(temporary_deduct=False)
        children_in_fact = qs.filter(
            grup=group, grup__status=GroupStatusEnum.FACT
        ).values_list("children_id", flat=True)

        children_in_plan = qs.filter(
            Q(grup__room_id__isnull=False, grup__room_id=group.room_id)
        ).filter(
            grup__status=GroupStatusEnum.PLAN
        ).exclude(
            children_id__in=qs.filter(
                grup__unit=group.unit,  grup__status=GroupStatusEnum.FACT
            ).values_list("children_id", flat=True)
        ).values_list("children_id", flat=True)

        children_ids.update(children_in_fact, children_in_plan)

    return Direct.objects.filter(
        status__code__in=(DRS.REGISTER, DRS.DOGOVOR),
        group__status__in=(GroupStatusEnum.FACT, GroupStatusEnum.PLAN),
    ).filter(
        declaration__children_id__in=children_ids
    ).distinct(
        'declaration__children'
    ).values_list(
        'declaration__children_id', 'declaration__children__date_of_birth'
    )
